@@ -158,30 +158,20 @@ class MockFixture:
             module, registering the patch to stop it later and returns the
             mock object resulting from the mock call.
             """
-            self._enforce_no_with_context(inspect.stack())
             p = mock_func(*args, **kwargs)
             mocked = p.start()
             self._patches.append(p)
             if hasattr(mocked, "reset_mock"):
                 self._mocks.append(mocked)
+                # check if `mocked` is actually a mock object, as depending on autospec or target
+                # parameters `mocked` can be anything
+                if hasattr(mocked, "__enter__"):
+                    mocked.__enter__.side_effect = ValueError(
+                        "Using mocker in a with context is not supported. "
+                        "https://github.com/pytest-dev/pytest-mock#note-about-usage-as-context-manager"
+                    )
             return mocked
 
-        def _enforce_no_with_context(self, stack):
-            """raises a ValueError if mocker is used in a with context"""
-            caller = stack[2]
-            frame = caller[0]
-            info = inspect.getframeinfo(frame)
-            if info.code_context is None:
-                # no source code available (#169)
-                return
-            code_context = " ".join(info.code_context).strip()
-
-            if code_context.startswith("with mocker."):
-                raise ValueError(
-                    "Using mocker in a with context is not supported. "
-                    "https://github.com/pytest-dev/pytest-mock#note-about-usage-as-context-manager"
-                )
-
         def object(self, *args, **kwargs):
             """API to mock.patch.object"""
             return self._start_patch(self.mock_module.patch.object, *args, **kwargs)
@@ -199,8 +189,7 @@ class MockFixture:
             return self._start_patch(self.mock_module.patch, *args, **kwargs)
 
 
-@pytest.yield_fixture
-def mocker(pytestconfig):
+def _mocker(pytestconfig):
     """
     return an object that has the same interface to the `mock` module, but
     takes care of automatically undoing all patches after each test method.
@@ -210,6 +199,13 @@ def mocker(pytestconfig):
     result.stopall()
 
 
+mocker = pytest.yield_fixture()(_mocker)  # default scope is function
+class_mocker = pytest.yield_fixture(scope="class")(_mocker)
+module_mocker = pytest.yield_fixture(scope="module")(_mocker)
+package_mocker = pytest.yield_fixture(scope="package")(_mocker)
+session_mocker = pytest.yield_fixture(scope="session")(_mocker)
+
+
 _mock_module_patches = []
 _mock_module_originals = {}
 
